@@ -10,7 +10,9 @@ setClass(
            particles = 'function',
            var.factor='numeric',
            ic.lag='integer',
-           cooling.factor='numeric',
+           cooling.type='character',
+           cooling.fraction='numeric',
+           method='character',
            random.walk.sd = 'numeric',
            conv.rec = 'matrix'
            )
