@@ -1,50 +1,3 @@
-## define the pomp class
-setClass(
-         'pomp',
-         representation(
-                        data = 'array',
-                        times = 'numeric',
-                        t0 = 'numeric',
-                        rprocess = 'function',
-                        dprocess = 'function',
-                        dmeasure = 'pomp.fun',
-                        rmeasure = 'pomp.fun',
-                        skeleton.type = 'character',
-                        skeleton = 'pomp.fun',
-                        skelmap.delta.t = 'numeric',
-                        initializer = 'function',
-                        states = 'array',
-                        params = 'numeric',
-                        covar = 'matrix',
-                        tcovar = 'numeric',
-                        obsnames = 'character',
-                        statenames = 'character',
-                        paramnames = 'character',
-                        covarnames = 'character',
-                        zeronames = 'character',
-                        has.trans = 'logical',
-                        par.trans = 'pomp.fun',
-                        par.untrans = 'pomp.fun',
-                        PACKAGE = 'character',
-                        userdata = 'list'
-                        )
-         )
-
-## this is the initial-condition setting function that is used by default
-## it simply finds all parameters in the vector 'params' that have a name ending in '.0'
-## and returns a vector with their values with names stripped of '.0'
-default.initializer <- function (params, t0, ...) {
-  ivpnames <- grep("\\.0$",names(params),value=TRUE)
-  if (length(ivpnames)<1)
-    stop("default initializer error: no parameter names ending in ",sQuote(".0")," found: see ",sQuote("pomp")," documentation")
-  x <- params[ivpnames]
-  names(x) <- sub("\\.0$","",ivpnames)
-  x
-}
-
-## as of version 0.37-1 'pomp' is a generic function
-setGeneric("pomp",function(data,...)standardGeneric("pomp"))
-
 ## basic constructor of the pomp class
 pomp.constructor <- function (data, times, t0, ..., rprocess, dprocess,
                               rmeasure, dmeasure, measurement.model,
@@ -89,7 +42,7 @@ pomp.constructor <- function (data, times, t0, ..., rprocess, dprocess,
   ## check t0
   if (!is.numeric(t0) || length(t0) > 1)
     stop("pomp error: the zero-time ",sQuote("t0")," must be a single number",call.=TRUE)
-  if (t0 > times[1])
+  if (t0 > times[1L])
     stop("pomp error: the zero-time ",sQuote("t0")," must occur no later than the first observation",call.=TRUE)
   storage.mode(t0) <- 'double'
   
@@ -306,8 +259,8 @@ measform2pomp <- function (formulae) {
     if (!inherits(formulae[[k]],"formula"))
       stop("pomp error: ",sQuote("measurement.model")," takes formulae as arguments",call.=FALSE)
   }
-  obsnames <- unlist(lapply(formulae,function(x)x[[2]]))
-  distrib <- lapply(formulae,function(x)as.character(x[[3]][[1]]))
+  obsnames <- unlist(lapply(formulae,function(x)x[[2L]]))
+  distrib <- lapply(formulae,function(x)as.character(x[[3L]][[1L]]))
   ddistrib <- lapply(distrib,function(x)paste0("d",x))
   rdistrib <- lapply(distrib,function(x)paste0("r",x))
   for (k in seq_len(nobs)) {
@@ -324,7 +277,7 @@ measform2pomp <- function (formulae) {
     if (inherits(res,'try-error'))
       stop("pomp error: random deviate function ",rdistrib[[k]]," not found")
   }
-  pred.args <- lapply(formulae,function(x)as.list(x[[3]][-1]))
+  pred.args <- lapply(formulae,function(x)as.list(x[[3L]][-1L]))
   dcalls <- vector(mode='list',length=nobs)
   rcalls <- vector(mode='list',length=nobs)
   for (k in seq_len(nobs)) {
