@@ -2,7 +2,8 @@
 
 simulate.internal <- function (object, nsim = 1, seed = NULL, params,
                                states = FALSE, obs = FALSE,
-                               times, t0, as.data.frame = FALSE, ...) {
+                               times, t0, as.data.frame = FALSE,
+                               .getnativesymbolinfo = TRUE, ...) {
 
   if (missing(times))
     times <- time(object,t0=FALSE)
@@ -44,11 +45,13 @@ simulate.internal <- function (object, nsim = 1, seed = NULL, params,
                       t0,
                       nsim,
                       obs,
-                      states
+                      states,
+                      .getnativesymbolinfo
                       ),
                 silent=FALSE
                 )
-
+  .getnativesymbolinfo <- FALSE
+  
   if (inherits(retval,'try-error'))
     stop(sQuote("simulate")," error",call.=FALSE)
 
@@ -59,14 +62,14 @@ simulate.internal <- function (object, nsim = 1, seed = NULL, params,
   if (as.data.frame) {
     if (obs && states) {
       dm <- dim(retval$obs)
-      nsim <- dm[2]
-      ntimes <- dm[3]
+      nsim <- dm[2L]
+      ntimes <- dm[3L]
       nm <- rownames(retval$obs)
-      dim(retval$obs) <- c(dm[1],prod(dm[-1]))
+      dim(retval$obs) <- c(dm[1L],prod(dm[-1L]))
       rownames(retval$obs) <- nm
       dm <- dim(retval$states)
       nm <- rownames(retval$states)
-      dim(retval$states) <- c(dm[1],prod(dm[-1]))
+      dim(retval$states) <- c(dm[1L],prod(dm[-1L]))
       rownames(retval$states) <- nm
       retval <- cbind(
                       as.data.frame(t(retval$obs)),
@@ -77,10 +80,10 @@ simulate.internal <- function (object, nsim = 1, seed = NULL, params,
       retval <- retval[order(retval$sim,retval$time),]
     } else if (obs || states) {
       dm <- dim(retval)
-      nsim <- dm[2]
-      ntimes <- dm[3]
+      nsim <- dm[2L]
+      ntimes <- dm[3L]
       nm <- rownames(retval)
-      dim(retval) <- c(dm[1],prod(dm[-1]))
+      dim(retval) <- c(dm[1L],prod(dm[-1L]))
       rownames(retval) <- nm
       retval <- as.data.frame(t(retval))
       retval$sim <- factor(seq_len(nsim))
@@ -110,4 +113,11 @@ simulate.internal <- function (object, nsim = 1, seed = NULL, params,
   retval
 }
 
-setMethod("simulate",signature=signature(object="pomp"),definition=simulate.internal)
+setMethod("simulate",signature=signature(object="pomp"),
+          definition=function (object, nsim = 1, seed = NULL, params,
+                               states = FALSE, obs = FALSE,
+                               times, t0, as.data.frame = FALSE, ...)
+          simulate.internal(object=object,nsim=nsim,seed=seed,params=params,
+                            states=states,obs=obs,times=times,t0=t0,
+                            as.data.frame=as.data.frame,...)
+          )
